@@ -53,7 +53,7 @@ class IndexedCorpus(interfaces.CorpusABC):
 
 
     @classmethod
-    def serialize(serializer, fname, corpus, id2word=None, index_fname=None, progress_cnt=None):
+    def serialize(serializer, fname, corpus, id2word=None, index_fname=None, progress_cnt=None, labels=None):
         """
         Iterate through the document stream `corpus`, saving the documents to `fname`
         and recording byte offset of each document. Save the resulting index
@@ -77,12 +77,19 @@ class IndexedCorpus(interfaces.CorpusABC):
             index_fname = fname + '.index'
 
         if progress_cnt is not None:
-            offsets = serializer.save_corpus(fname, corpus, id2word, progress_cnt=progress_cnt)
+            if labels is not None:
+                offsets = serializer.save_corpus(fname, corpus, id2word, labels=labels, progress_cnt=progress_cnt)
+            else:
+                offsets = serializer.save_corpus(fname, corpus, id2word, progress_cnt=progress_cnt)
         else:
-            offsets = serializer.save_corpus(fname, corpus, id2word)
+            if labels is not None:
+                offsets = serializer.save_corpus(fname, corpus, id2word, labels=labels)
+            else:
+                offsets = serializer.save_corpus(fname, corpus, id2word)
+
         if offsets is None:
-            raise NotImplementedError("called serialize on class %s which \
-            doesn't support indexing!" % serializer.__name__)
+            raise NotImplementedError("called serialize on class %s which doesn't support indexing!" %
+                serializer.__name__)
 
         # store offsets persistently, using pickle
         logger.info("saving %s index to %s" % (serializer.__name__, index_fname))
