@@ -103,7 +103,7 @@ class TestDictionary(unittest.TestCase):
         self.assertEqual(d.dfs, expected)
 
     def test_saveAsText_and_loadFromText(self):
-        """ `Dictionary` can be saved as textfile and loaded again from textfile. """
+        """`Dictionary` can be saved as textfile and loaded again from textfile. """
         tmpf = get_tmpfile('dict_test.txt')
         d = Dictionary(self.texts)
         d.save_as_text(tmpf)
@@ -113,6 +113,41 @@ class TestDictionary(unittest.TestCase):
         d_loaded = Dictionary.load_from_text(get_tmpfile('dict_test.txt'))
         self.assertNotEqual(d_loaded, None)
         self.assertEqual(d_loaded.token2id, d.token2id)
+
+    def test_from_corpus(self):
+        """build `Dictionary` from an existing corpus"""
+
+        documents = ["Human machine interface for lab abc computer applications",
+                "A survey of user opinion of computer system response time",
+                "The EPS user interface management system",
+                "System and human system engineering testing of EPS",
+                "Relation of user perceived response time to error measurement",
+                "The generation of random binary unordered trees",
+                "The intersection graph of paths in trees",
+                "Graph minors IV Widths of trees and well quasi ordering",
+                "Graph minors A survey"]
+        stoplist = set('for a of the and to in'.split())
+        texts = [[word for word in document.lower().split() if word not in stoplist]
+                for document in documents]
+
+        # remove words that appear only once
+        all_tokens = sum(texts, [])
+        tokens_once = set(word for word in set(all_tokens) if all_tokens.count(word) == 1)
+        texts = [[word for word in text if word not in tokens_once]
+                for text in texts]
+        dictionary = Dictionary(texts)
+        corpus = [dictionary.doc2bow(text) for text in texts]
+        dictionary_from_corpus = Dictionary.from_corpus(corpus)
+
+        #we have to compare values, because in creating dictionary from corpus
+        #informations about words are lost
+        dict_token2id_vals = sorted(dictionary.token2id.values())
+        dict_from_corpus_vals = sorted(dictionary_from_corpus.token2id.values())
+        self.assertEqual(dict_token2id_vals, dict_from_corpus_vals)
+        self.assertEqual(dictionary.dfs, dictionary_from_corpus.dfs)
+        self.assertEqual(dictionary.num_docs, dictionary_from_corpus.num_docs)
+        self.assertEqual(dictionary.num_pos, dictionary_from_corpus.num_pos)
+        self.assertEqual(dictionary.num_nnz, dictionary_from_corpus.num_nnz)
 #endclass TestDictionary
 
 
