@@ -22,15 +22,12 @@ import itertools
 import tempfile
 from functools import wraps # for `synchronous` function lock
 from htmlentitydefs import name2codepoint as n2cp # for `decode_htmlentities`
-import threading, time
-from Queue import Queue, Empty
+import multiprocessing
 import shutil
 
-
 try:
     from pattern.en import parse
-    from multiprocessing import Process, Queue as PQueue, cpu_count
-    logger.info("'pattern' package found; utils.Lemmatizater is available for English")
+    logger.info("'pattern' package found; utils.lemmatize() is available for English")
     HAS_PATTERN = True
 except ImportError:
     HAS_PATTERN = False
@@ -383,29 +380,33 @@ def decode_htmlentities(text):
         return text
 
 
-def chunkize_serial(corpus, chunksize):
+def chunkize_serial(iterable, chunksize, as_numpy=False):
     """
-    Split a stream of values into smaller chunks.
-    Each chunk is of length `chunksize`, except the last one which may be smaller.
-    A once-only input stream (`corpus` from a generator) is ok.
-
-    >>> for chunk in chunkize_serial(xrange(10), 4): print list(chunk)
-    [0, 1, 2, 3]
-    [4, 5, 6, 7]
-    [8, 9]
+    Return elements from the iterable in `chunksize`-ed lists. The last returned
+    element may be smaller (if length of collection is not divisible by `chunksize`).
 
+    >>> print list(grouper(xrange(10), 3))
+    [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]
     """
-    if chunksize <= 0:
-        raise ValueError("chunk size must be greater than zero")
-    i = (val for val in corpus) # create generator
+    import numpy
+    it = iter(iterable)
     while True:
-        chunk = [list(itertools.islice(i, int(chunksize)))] # consume `chunksize` items from the generator
-        if not chunk[0]: # generator empty?
+        if as_numpy:
+            # convert each document to a 2d numpy array (~6x faster when transmitting
+            # chunk data over the wire, in Pyro)
+            wrapped_chunk = [[numpy.array(doc) for doc in itertools.islice(it, int(chunksize))]]
+        else:
+            wrapped_chunk = [list(itertools.islice(it, int(chunksize)))]
+        if not wrapped_chunk[0]:
             break
-        yield chunk.pop()
+        # memory opt: wrap the chunk and then pop(), to avoid leaving behind a dangling reference
+        yield wrapped_chunk.pop()
+
+grouper = chunkize_serial
 
 
-def chunkize(corpus, chunksize, maxsize=0):
+
+def chunkize(corpus, chunksize, maxsize=0, as_numpy=False):
     """
     Split a stream of values into smaller chunks.
     Each chunk is of length `chunksize`, except the last one which may be smaller.
@@ -414,11 +415,11 @@ def chunkize(corpus, chunksize, maxsize=0):
 
     If `maxsize > 1`, don't wait idly in between successive chunk `yields`, but
     rather keep filling a short queue (of size at most `maxsize`) with forthcoming
-    chunks in advance. This is realized by starting a separate thread, and is
+    chunks in advance. This is realized by starting a separate process, and is
     meant to reduce I/O delays, which can be significant when `corpus` comes
     from a slow medium (like harddisk).
 
-    If `maxsize==0`, don't fool around with threads and simply yield the chunksize
+    If `maxsize==0`, don't fool around with parallelism and simply yield the chunksize
     via `chunkize_serial()` (no I/O optimizations).
 
     >>> for chunk in chunkize(xrange(10), 4): print chunk
@@ -427,43 +428,54 @@ def chunkize(corpus, chunksize, maxsize=0):
     [8, 9]
 
     """
-    class InputQueue(threading.Thread):
-        """
-        Help class for threaded `chunkize()`.
-        """
-        def __init__(self, q, corpus, chunksize, maxsize):
+    class InputQueue(multiprocessing.Process):
+        def __init__(self, q, corpus, chunksize, maxsize, as_numpy):
             super(InputQueue, self).__init__()
             self.q = q
             self.maxsize = maxsize
             self.corpus = corpus
             self.chunksize = chunksize
+            self.as_numpy = as_numpy
 
         def run(self):
-            import numpy # don't clutter the global namespace with a dependency on numpy
-            i = (val for val in self.corpus) # create generator
+            if self.as_numpy:
+                import numpy # don't clutter the global namespace with a dependency on numpy
+            it = iter(self.corpus)
             while True:
-                # HACK XXX convert documents to numpy arrays, to save memory.
-                # This also gives a scipy warning at runtime:
-                # "UserWarning: indices array has non-integer dtype (float64)"
-                chunk = [numpy.asarray(doc) for doc in itertools.islice(i, self.chunksize)] # consume `chunksize` items from the generator
-                if not chunk: # generator empty?
+                chunk = itertools.islice(it, self.chunksize)
+                if self.as_numpy:
+                    # HACK XXX convert documents to numpy arrays, to save memory.
+                    # This also gives a scipy warning at runtime:
+                    # "UserWarning: indices array has non-integer dtype (float64)"
+                    wrapped_chunk = [[numpy.asarray(doc) for doc in chunk]]
+                else:
+                    wrapped_chunk = [list(chunk)]
+
+                if not wrapped_chunk[0]:
+                    self.q.put(None, block=True)
                     break
-                logger.info("prepared another chunk of %i documents (qsize=%i)" %
-                            (len(chunk), self.q.qsize()))
-                self.q.put(chunk, block=True)
+
+                try:
+                    qsize = self.q.qsize()
+                except NotImplementedError:
+                    qsize = '?'
+                logger.debug("prepared another chunk of %i documents (qsize=%s)" %
+                            (len(wrapped_chunk[0]), qsize))
+                self.q.put(wrapped_chunk.pop(), block=True)
     #endclass InputQueue
 
     assert chunksize > 0
 
     if maxsize > 0:
-        q = Queue(maxsize=maxsize)
-        thread = InputQueue(q, corpus, chunksize, maxsize=maxsize)
-        thread.start()
-        while thread.isAlive() or not q.empty():
-            try:
-                yield q.get(block=True, timeout=1)
-            except Empty:
-                pass
+        q = multiprocessing.Queue(maxsize=maxsize)
+        worker = InputQueue(q, corpus, chunksize, maxsize=maxsize, as_numpy=as_numpy)
+        worker.daemon = True
+        worker.start()
+        while True:
+            chunk = [q.get(block=True)]
+            if chunk[0] is None:
+                break
+            yield chunk.pop()
     else:
         for chunk in chunkize_serial(corpus, chunksize):
             yield chunk
@@ -513,29 +525,6 @@ def randfname(prefix='gensim'):
     return os.path.join(tempfile.gettempdir(), prefix + randpart)
 
 
-def grouper(iterable, chunksize, as_numpy=False):
-    """
-    Return elements from the iterable in `chunksize`-ed lists. The last returned
-    element may be smaller (if length of collection is not divisible by `chunksize`).
-
-    >>> print list(grouper(xrange(10), 3))
-    [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]
-    """
-    import numpy
-    it = iter(iterable)
-    while True:
-        if as_numpy:
-            # convert each document to a 2d numpy array (~6x faster when transmitting
-            # chunk data over the wire, in Pyro)
-            wrapped_chunk = [[numpy.array(doc) for doc in itertools.islice(it, int(chunksize))]]
-        else:
-            wrapped_chunk = [list(itertools.islice(it, int(chunksize)))]
-        if not wrapped_chunk[0]:
-            break
-        # memory opt: wrap the chunk and then pop(), to avoid leaving behind a dangling reference
-        yield wrapped_chunk.pop()
-
-
 def upload_chunked(server, docs, chunksize=1000, preprocess=None):
     """
     Memory-friendly upload of documents to a SimServer (or Pyro SimServer proxy).
@@ -624,87 +613,4 @@ if HAS_PATTERN:
                         lemma += "/" + tag[:2]
                         result.append(lemma.encode('utf8'))
         return result
-
-
-    def lemmatize_queue(qin, qout):
-        while True:
-            seq_id, content = qin.get()
-            if seq_id is None:
-                return
-            qout.put((seq_id, lemmatize(content)))
-
-
-    class Lemmatizer(object):
-        """
-        Wraps the lemmatize() fnc so that input can be processed in parallel, to
-        speed things up.
-
-        Main methods are `feed(content)`, which puts the content in queue for
-        lemmatization, and `read()`, which returns lemmatized content when it's ready.
-
-        Note that the order of content entered and read back isn't necessarily the same!
-        Use the sequence id returned from `feed`/`read` to match input to output.
-
-        This class is NOT thread-safe.
-
-        Example:
-
-        >>> from gensim import utils
-        >>> lm = utils.Lemmatizer()
-        >>> docs = ['quick brown fox', 'slow yellow tortoise', 'document nr. three']
-        >>> for doc in docs:
-        >>>     print lm.feed(doc) # feed the documents into the parser, to be processed in parallel
-        -2939683893812816931
-        5130064701692321566
-        -1045879559712133929
-        >>> lm.read()
-        (5130064701692321566, ['slow/JJ', 'yellow/JJ', 'tortoise/NN'])
-        >>> lm.read()
-        (-2939683893812816931, ['quick/JJ', 'brown/JJ', 'fox/NN'])
-        >>> lm.read()
-        (-1045879559712133929, ['document/NN', 'nr/NN'])
-
-        """
-        FEED_MAX_QUEUE = 1000 # block after the parsing queue has reached this length -- new feed()/read() calls will have to wait
-
-        def __init__(self, num_workers=cpu_count()):
-            logger.info("initializing lemmatizer with %i processes" % num_workers)
-            self.num_workers = num_workers
-            self.qin = PQueue(maxsize=Lemmatizer.FEED_MAX_QUEUE)
-            self.qout = PQueue(maxsize=Lemmatizer.FEED_MAX_QUEUE)
-            # start up processes that will be parsing in parallel
-            self.prcs = []
-            for _ in xrange(self.num_workers):
-                prc = Process(target=lemmatize_queue, args=(self.qin, self.qout))
-                prc.daemon = True
-                prc.start()
-                self.prcs.append(prc)
-
-        def feed(self, content):
-            """Place a document (text) into the "to be lemmatized" queue"""
-            seq_id = content.__hash__()
-            self.qin.put((seq_id, content))
-            return seq_id
-
-        def read(self):
-            """Return one lemmatized document; blocks if no document ready yet."""
-            seq_id, lemmas = self.qout.get()
-            if seq_id is None:
-                logger.warning('lemmatizer failed for input #%s' % seq_id)
-            return seq_id, lemmas
-
-        def has_results(self):
-            """The next call to read() won't block (not thread-safe!)."""
-            return not self.qout.empty()
-
-        def __del__(self):
-            try:
-                for prc in self.prcs:
-                    prc.terminate()
-                logger.info("terminated %i lemmatizer processes" % self.num_workers)
-            except:
-                # ignore errors at interpreter tear-down
-                pass
-
-    lemmatizer = Lemmatizer()
 #endif HAS_PATTERN
